@@ -16,10 +16,10 @@ limitations under the License.
 #ifndef XLA_SERVICE_CONDITIONAL_SIMPLIFIER_H_
 #define XLA_SERVICE_CONDITIONAL_SIMPLIFIER_H_
 
+#include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
 #include "xla/hlo/ir/hlo_module.h"
-#include "xla/service/hlo_pass_interface.h"
-#include "xla/statusor.h"
+#include "xla/hlo/pass/hlo_pass_interface.h"
 
 namespace xla {
 
